@@ -344,7 +344,8 @@ class _VNCSocketListener(threading.Thread):
 
     def run(self):
         """Used by the thread to run the listener."""
-        server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        family = socket.AF_INET6 if ':' in self.local_ip else socket.AF_INET
+        server = socket.socket(family, socket.SOCK_STREAM)
         server.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
         server.bind((self.local_ip, self.remote_port))
         LOG.info(_("VNCSocket Listener Listening on ip=%(ip)s port=%(port)s") %
