@@ -56,7 +56,7 @@ class TestValidator(testtools.TestCase):
                       env='AIX/Linux', proc_compat='Default', srr_enabled=True,
                       min_vcpus=1, des_vcpus=2, max_vcpus=4,
                       min_proc_units=0.1, max_proc_units=1.0, pool_id=None,
-                      exp_factor=0.0, ame_enabled=False):
+                      exp_factor=0.0, ame_enabled=False, ppt_ratio=None):
             lpar_w = mock.MagicMock()
             # name, states, env, etc.
             lpar_w.name = name
@@ -85,6 +85,7 @@ class TestValidator(testtools.TestCase):
             lpar_w.mem_config.min = min_mem
             lpar_w.mem_config.max = max_mem
             lpar_w.mem_config.exp_factor = exp_factor
+            lpar_w.mem_config.ppt_ratio = ppt_ratio
             # Can Modify
             if (state != bp.LPARState.NOT_ACTIVATED
                and rmc_state != bp.RMCState.ACTIVE):
@@ -164,6 +165,8 @@ class TestValidator(testtools.TestCase):
                                     ame_enabled=True)
         self.lpar_ame_3 = _bld_lpar(name='ame_3', exp_factor=3.0,
                                     ame_enabled=True)
+        self.lpar_ppt_1 = _bld_lpar(name='ppt_1', ppt_ratio=4)
+        self.lpar_ppt_2 = _bld_lpar(name='ppt_2', ppt_ratio=2)
 
     def test_validator(self):
         # Test desired proc units > host avail proc units fails for shared
@@ -248,6 +251,14 @@ class TestValidator(testtools.TestCase):
         vldr = vldn.LPARWrapperValidator(self.lpar_ame_2, self.mngd_sys,
                                          cur_lpar_w=self.lpar_1_proc)
         self.assertRaises(vldn.ValidatorException, vldr.validate_all)
+        # Test changing PPT ratio fails during active resize
+        vldr = vldn.LPARWrapperValidator(self.lpar_ppt_1, self.mngd_sys,
+                                         cur_lpar_w=self.lpar_ppt_2)
+        self.assertRaises(vldn.ValidatorException, vldr.validate_all)
+        # Test not changing PPT ratio passes during active resize
+        vldr = vldn.LPARWrapperValidator(self.lpar_1_proc, self.mngd_sys,
+                                         cur_lpar_w=self.lpar_ppt_2)
+        vldr.validate_all()
         # Test resizing lpar from defaultSPP to non-defaultSPP passes
         vldr = vldn.LPARWrapperValidator(self.lpar_non_default_spp,
                                          self.mngd_sys,
@@ -371,3 +382,19 @@ class TestValidator(testtools.TestCase):
             proc_vldr.validate()
             self.assertTrue(active_resize_checks.called,
                             'Active resize validations not performed.')
+
+    @mock.patch('pypowervm.utils.validation.ProcValidator.validate')
+    @mock.patch('pypowervm.utils.validation.MemValidator.validate')
+    def test_validator_check_dlpar(self, mem_val_validate, proc_val_validate):
+        vldr = vldn.LPARWrapperValidator(self.lpar_1_proc, self.mngd_sys,
+                                         cur_lpar_w=self.lpar_no_rmc)
+        vldr.validate_all(check_dlpar=False)
+        mem_val_validate.assert_called_once_with(check_dlpar=False)
+        proc_val_validate.assert_called_once_with(check_dlpar=False)
+        mem_val_validate.reset_mock()
+        proc_val_validate.reset_mock()
+        vldr = vldn.LPARWrapperValidator(self.lpar_running, self.mngd_sys,
+                                         cur_lpar_w=self.lpar_running)
+        vldr.validate_all()
+        mem_val_validate.assert_called_once_with(check_dlpar=True)
+        proc_val_validate.assert_called_once_with(check_dlpar=True)
