@@ -50,6 +50,7 @@ _IBMi_RESTRICTEDIO_CAP = u.xpath(
 _SIMP_REMOTE_RESTART_CAP = u.xpath(
     _SYS_CAPABILITIES, 'PowerVMLogicalPartitionSimplifiedRemoteRestartCapable')
 _AME_CAP = u.xpath(_SYS_CAPABILITIES, 'ActiveMemoryExpansionCapable')
+_PPT_CAP = u.xpath(_SYS_CAPABILITIES, 'CustomPhysicalPageTableRatioCapable')
 _AIX_CAP = u.xpath(_SYS_CAPABILITIES, 'AIXCapable')
 _IBMi_CAP = u.xpath(_SYS_CAPABILITIES, 'IBMiCapable')
 _LINUX_CAP = u.xpath(_SYS_CAPABILITIES, 'LinuxCapable')
@@ -62,6 +63,8 @@ _DYN_SRR_CAP = u.xpath(
 _IBMi_NATIVE_IO_CAP = u.xpath(_SYS_CAPABILITIES, 'IBMiNativeIOCapable')
 _DISABLE_SECURE_BOOT_CAP = u.xpath(
     _SYS_CAPABILITIES, 'DisableSecureBootCapable')
+_IOSLOT_OWNER_ASSMT_CAP = u.xpath(
+    _SYS_CAPABILITIES, 'IOSlotOwnerAssignmentCapable')
 
 _CAPABILITY_MAP = {
     'active_lpar_mobility_capable': {
@@ -77,6 +80,8 @@ _CAPABILITY_MAP = {
         'prop': _IBMi_RESTRICTEDIO_CAP, 'default': False},
     'simplified_remote_restart_capable': {
         'prop': _SIMP_REMOTE_RESTART_CAP, 'default': False},
+    'physical_page_table_ratio_capable': {
+        'prop': _PPT_CAP, 'default': False},
     'active_memory_expansion_capable': {
         'prop': _AME_CAP, 'default': False},
     # aix_capable defaults to True for backward compat (that is what we
@@ -99,6 +104,8 @@ _CAPABILITY_MAP = {
         'prop': _IBMi_NATIVE_IO_CAP, 'default': False},
     'disable_secure_boot_capable': {
         'prop': _DISABLE_SECURE_BOOT_CAP, 'default': False},
+    'ioslot_owner_assignment_capable': {
+        'prop': _IOSLOT_OWNER_ASSMT_CAP, 'default': False},
 }
 
 _SYS_MEM_CONFIG = 'AssociatedSystemMemoryConfiguration'
@@ -108,6 +115,7 @@ _MEMORY_CONFIGURABLE = u.xpath(_SYS_MEM_CONFIG, 'ConfigurableSystemMemory')
 _MEMORY_REGION_SIZE = u.xpath(_SYS_MEM_CONFIG, 'MemoryRegionSize')
 _SYS_FIRMWARE_MEM = u.xpath(_SYS_MEM_CONFIG, 'MemoryUsedByHypervisor')
 _PAGE_TABLE_RATIO = u.xpath(_SYS_MEM_CONFIG, 'DefaultHardwarePageTableRatio')
+_DEFAULT_PPT_RATIO = u.xpath(_SYS_MEM_CONFIG, 'DefaultPhysicalPageTableRatio')
 
 # Migration Constants
 _SYS_PROC_CONFIG = 'AssociatedSystemProcessorConfiguration'
@@ -162,6 +170,10 @@ _IOSLOT_ROOT = 'IOSlot'
 _IOSLOT_BUS_GRP_REQ = 'BusGroupingRequired'
 _IOSLOT_DESC = 'Description'
 _IOSLOT_FEAT_CODES = 'FeatureCodes'
+_IOSLOT_PART_ID = 'PartitionID'
+_IOSLOT_PART_UUID = 'PartitionUUID'
+_IOSLOT_PART_NAME = 'PartitionName'
+_IOSLOT_PART_TYPE = 'PartitionType'
 _IOSLOT_PCI_CLASS = 'PCIClass'
 _IOSLOT_PCI_DEV_ID = 'PCIDeviceID'
 _IOSLOT_PCI_SUB_DEV_ID = 'PCISubsystemDeviceID'
@@ -268,6 +280,12 @@ class System(ewrap.EntryWrapper):
     def page_table_ratio(self):
         return self._get_val_int(_PAGE_TABLE_RATIO, 0)
 
+    @property
+    def default_ppt_ratio(self):
+        # Platforms that don't provide the default PPTR from REST
+        # have a default value of 6 (1:4096).
+        return self._get_val_int(_DEFAULT_PPT_RATIO, 6)
+
     @property
     def host_ip_address(self):
         prop = _HOST_IP_ADDRESS
@@ -414,6 +432,40 @@ class IOSlot(ewrap.ElementWrapper):
     def feat_codes(self):
         return self._get_val_int(_IOSLOT_FEAT_CODES)
 
+    @property
+    def part_id(self):
+        """Short ID of the partition to which the slot is assigned.
+
+        None if the slot is unassigned.
+        """
+        return self._get_val_int(_IOSLOT_PART_ID)
+
+    @property
+    def part_uuid(self):
+        """UUID of the partition to which the slot is assigned.
+
+        None if the slot is unassigned.
+        """
+        return self._get_val_str(_IOSLOT_PART_UUID)
+
+    @property
+    def part_name(self):
+        """String name of the partition to which the slot is assigned.
+
+        None if the slot is unassigned.
+        """
+        return self._get_val_str(_IOSLOT_PART_NAME)
+
+    @property
+    def part_type(self):
+        """String type of the partition to which the slot is assigned.
+
+        May be compared with base_partition.LPARType enum values.
+
+        None if the slot is unassigned.
+        """
+        return self._get_val_str(_IOSLOT_PART_TYPE)
+
     @property
     def pci_class(self):
         return self._get_val_int(_IOSLOT_PCI_CLASS)
