@@ -146,6 +146,9 @@ _VADPT_SLOT_NUM = 'VirtualSlotNumber'
 _VADPT_ENABLED = 'Enabled'
 _VADPT_ALLOWED_MAC = _TA_ALLOWED_MAC
 _VADPT_MAC_ADDR = _TA_MAC
+_VADPT_IP_ADDR = 'IPAddress'
+_VADPT_SUBNET_MASK = 'SubnetMask'
+_VADPT_GATEWAY = 'Gateway'
 _VADPT_PVID = _PVID
 _VADPT_QOS_PRI = 'QualityOfServicePriority'
 _VADPT_QOS_PRI_ENABLED = _TA_QOS_PRI
@@ -170,8 +173,8 @@ _VADPT_EL_ORDER = (
     _VADPT_QOS_PRI_ENABLED, _VADPT_TAGGED_VLANS, _VADPT_TAGGED_VLAN_SUPPORT,
     _VADPT_VSI_MANAGER_ID, _VADPT_VSI_TYPE_ID, _VADPT_VSWITCH,
     _VADPT_VSWITCH_ID, _VADPT_VSI_TYPE_VERSION, _VADPT_DEV_NAME,
-    _VADPT_TRUNK_PRI, _VADPT_VNETS, _VADPT_MTU, _VADPT_OVS_BRIDGE,
-    _VADPT_OVS_EXT_IDS)
+    _VADPT_TRUNK_PRI, _VADPT_MTU, _VADPT_OVS_BRIDGE, _VADPT_OVS_EXT_IDS,
+    _VADPT_VNETS)
 
 
 class VSwitchMode(object):
@@ -1112,12 +1115,12 @@ class CNA(ewrap.EntryWrapper):
             cna._dev_name(dev_name)
 
         if ovs_bridge is not None:
-            cna._ovs_bridge(ovs_bridge)
+            cna.ovs_bridge = ovs_bridge
             if ovs_ext_ids is not None:
-                cna._ovs_ext_ids(ovs_ext_ids)
+                cna.ovs_ext_ids = ovs_ext_ids
 
         if configured_mtu is not None:
-            cna._configured_mtu(configured_mtu)
+            cna.configured_mtu = configured_mtu
 
         # If a trunk priority is specified, set it. It will make this CNA
         # build out a trunk adapter. However, if it is not specified, we
@@ -1361,7 +1364,8 @@ class CNA(ewrap.EntryWrapper):
         """
         return self._get_val_int(_VADPT_MTU)
 
-    def _configured_mtu(self, value):
+    @configured_mtu.setter
+    def configured_mtu(self, value):
         self.set_parm_value(_VADPT_MTU, value, attrib=c.ATTR_KSV160)
 
     @property
@@ -1369,7 +1373,8 @@ class CNA(ewrap.EntryWrapper):
         """The Open vSwitch bridge it is connected to.  Otherwise None."""
         return self._get_val_str(_VADPT_OVS_BRIDGE)
 
-    def _ovs_bridge(self, value):
+    @ovs_bridge.setter
+    def ovs_bridge(self, value):
         self.set_parm_value(_VADPT_OVS_BRIDGE, value, attrib=c.ATTR_KSV160)
 
     @property
@@ -1385,9 +1390,40 @@ class CNA(ewrap.EntryWrapper):
         """
         return self._get_val_str(_VADPT_OVS_EXT_IDS)
 
-    def _ovs_ext_ids(self, value):
+    @ovs_ext_ids.setter
+    def ovs_ext_ids(self, value):
         self.set_parm_value(_VADPT_OVS_EXT_IDS, value, attrib=c.ATTR_KSV160)
 
+    @ewrap.Wrapper.xag_property(c.XAG.ADV)
+    def ip_address(self):
+        """Returns the IP Address of the network interface.
+
+        Typical format would be: 255.255.255.255 (IPv4)
+        and ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff (IPv6)
+        or other short forms of IPv6 address
+        """
+        return self._get_val_str(_VADPT_IP_ADDR)
+
+    @ewrap.Wrapper.xag_property(c.XAG.ADV)
+    def subnet_mask(self):
+        """Returns the subnet mask of the network interface.
+
+        Typical format would be: 255.255.255.0 (IPv4)
+        and ffff:ffff:ffff:ffff:: (IPv6)
+        or other forms of IPv6 address
+        """
+        return self._get_val_str(_VADPT_SUBNET_MASK)
+
+    @ewrap.Wrapper.xag_property(c.XAG.ADV)
+    def gateway(self):
+        """Returns the gateway of the network interface.
+
+        Typical format would be: 10.0.0.1 (IPv4)
+        and cafe::1 (IPv6)
+        or other forms of IPv6 address
+        """
+        return self._get_val_str(_VADPT_GATEWAY)
+
 
 @ewrap.ElementWrapper.pvm_type(_SEA_ETH_BACK_DEV, has_metadata=True,
                                child_order=_SEA_EBD_ORDER)
